
/**
  ******************************************************************************
  * @file    nema_hal.c
  * @author  MCD Application Team
  * @brief   NemaGFX Interfaces and Platform Specific APIs with FreeRTOS CMSISV2
  *          support.
  *          This file provides NemaGFX functions to manage the following
  *          functionalities of the NemaGFX library:
  *           + Initialization and de-initialization functions
  *           + GPU2D Registers access operation functions
  *           + Buffer Creation, Destruction, MMAP and Flusing functions
  *           + Interrupt management functions
  *           + Memory management functions
  *           + Locking/Unlocking resources functions
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */

#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>

#include <nema_sys_defs.h>
#include <nema_core.h>

#include <assert.h>
#include <string.h>

#include "ewconfig.h"
#include "ewrte.h"
#include "ew_bsp_os.h"

#include <platform_pool_config.h>

static XSemaphoreHandle  AcceleratorSemaphoreId;

#include <stm32h7rsxx_hal.h>

#include <cmsis_os2.h>

#include "tsi_malloc.h"

#define RING_SIZE 1024

static nema_ringbuffer_t ring_buffer_str;
volatile static int last_cl_id = -1;
extern GPU2D_HandleTypeDef hgpu2d;

static osMutexId_t nema_flush_mutex = NULL;

#if (USE_HAL_GPU2D_REGISTER_CALLBACKS == 1)
static void GPU2D_CommandListCpltCallback(GPU2D_HandleTypeDef *hgpu2d, uint32_t CmdListID)
#else /* USE_HAL_GPU2D_REGISTER_CALLBACKS = 0 */
void HAL_GPU2D_CommandListCpltCallback(GPU2D_HandleTypeDef *hgpu2d, uint32_t CmdListID)
#endif /* USE_HAL_GPU2D_REGISTER_CALLBACKS = 1 */
{
    /* Prevent unused argument(s) compilation warning */
    UNUSED(hgpu2d);

    last_cl_id = CmdListID;

    EwBspOsSemaphoreRelease( AcceleratorSemaphoreId );

}

int32_t nema_sys_init(void)
{
    int ret;

#if (USE_HAL_GPU2D_REGISTER_CALLBACKS == 1)
    /* Register Command List Comlete Callback */
    HAL_GPU2D_RegisterCommandListCpltCallback(&hgpu2d, GPU2D_CommandListCpltCallback);
#endif /* USE_HAL_GPU2D_REGISTER_CALLBACKS = 1 */


    /* create the accelerator semaphore */
    AcceleratorSemaphoreId = EwBspOsSemaphoreCreate( 1, 1 );

    /* initially take the accelerator token for the first DMA2D transfer */
    EwBspOsSemaphoreWait( AcceleratorSemaphoreId, 1000 );

    nema_flush_mutex = osMutexNew(NULL);

    /* Initialise Mem Space */
    tsi_malloc_init_pool(0, (void*)nemagfx_pool_mem, (uintptr_t)nemagfx_pool_mem, NEMA_BYTE_POOL_SIZE, 1);

    /* Allocate ring_buffer memory */
    ring_buffer_str.bo = nema_buffer_create(RING_SIZE);
    assert(ring_buffer_str.bo.base_virt);

    /* Initialize Ring Buffer */
    ret = nema_rb_init(&ring_buffer_str, 1);
    if (ret < 0)
    {
        return ret;
    }

    /* Reset last_cl_id counter */
    last_cl_id = 0;

    return 0;
}

int nema_wait_irq(void)
{
  EwBspOsSemaphoreWait( AcceleratorSemaphoreId, 1000 );

    return 0;
}

int nema_wait_irq_cl(int cl_id)
{
    while (last_cl_id < cl_id)
    {
        (void)nema_wait_irq();
    }

    return 0;
}

int nema_wait_irq_brk(int brk_id)
{
    while (nema_reg_read(GPU2D_BREAKPOINT) == 0U)
    {
        (void)nema_wait_irq();
    }

    return 0;
}

uint32_t nema_reg_read(uint32_t reg)
{
    return HAL_GPU2D_ReadRegister(&hgpu2d, reg);
}

void nema_reg_write(uint32_t reg, uint32_t value)
{
    HAL_GPU2D_WriteRegister(&hgpu2d, reg, value);
}

nema_buffer_t nema_buffer_create(int size)
{
    nema_buffer_t bo;

    memset(&bo, 0, sizeof(bo));

    nema_mutex_lock(MUTEX_MALLOC);

    bo.base_virt = tsi_malloc(size);

    if ( !bo.base_virt )
      EwPrint( "nema_buffer_create() - out of memory!!!\n" );
    assert(bo.base_virt);

    bo.base_phys = (uint32_t)bo.base_virt;
    bo.size      = size;

    nema_mutex_unlock(MUTEX_MALLOC);

    return bo;
}

nema_buffer_t nema_buffer_create_pool(int pool, int size)
{
    nema_buffer_t bo;

    memset(&bo, 0, sizeof(bo));

    nema_mutex_lock(MUTEX_MALLOC); /* tsi_malloc isn't thread-safe */

    bo.base_virt = tsi_malloc_pool(pool, size);

    if ( !bo.base_virt )
      EwPrint( "nema_buffer_create_pool() - out of memory!!!\n" );
    assert(bo.base_virt);

    bo.base_phys = (uintptr_t)bo.base_virt;
    bo.size      = size;
    bo.fd        = pool; /* use fd during the memory free */

    nema_mutex_unlock(MUTEX_MALLOC);

    return bo;
}

void *nema_buffer_map(nema_buffer_t *bo)
{
    return bo->base_virt;
}

void nema_buffer_unmap(nema_buffer_t *bo)
{
    /* Prevent unused argument(s) compilation warning */
    UNUSED(bo);
}

void nema_buffer_destroy(nema_buffer_t *bo)
{
    if (bo->fd == -1)
    {
        return; /* Buffer weren't allocated! */
    }

    tsi_free(bo->base_virt);

    bo->base_virt = (void *)0;
    bo->base_phys = 0;
    bo->size      = 0;
    bo->fd        = -1; /* Buffer not allocated */
}

uintptr_t nema_buffer_phys(nema_buffer_t *bo)
{
    return bo->base_phys;
}

void nema_buffer_flush(nema_buffer_t * bo)
{

    osMutexAcquire(nema_flush_mutex, 0xFFFFFFFF);

    SCB_CleanInvalidateDCache_by_Addr((uint32_t *)bo->base_virt, bo->size);

    osMutexRelease(nema_flush_mutex);
}

void nema_host_free(void *ptr)
{
    if (ptr)
    {
        tsi_free(ptr);
    }
}

void* nema_host_malloc(unsigned size)
{
    void *ptr = NULL;

    ptr = tsi_malloc(size);

    if ( !ptr )
      EwPrint( "nema_host_malloc() - out of memory!!!\n" );
    assert(ptr);

    return ptr;
}

int nema_mutex_lock(int mutex_id)
{
    int ret = 0;

    UNUSED(mutex_id);

    return ret;
}

int nema_mutex_unlock(int mutex_id)
{
    int ret = 0;

    UNUSED(mutex_id);
    return ret;
}
